#include <htc.h>
#include "i2c.h"

__CONFIG( UNPROTECT & BOREN & MCLRDIS & PWRTEN & WDTDIS & INTCLK );
#define _XTAL_FREQ 4000000

//RotaryEncoderInputs
#define PHASEA GPIO0
#define PHASEB GPIO1

//These are defined in "i2c.h"
//SCL:GPIO5
//SDA:GPIO2

//i2c constants
#define I2CADDR 0x50
#define I2CREAD 1
#define I2CWRITE 0

//registers
#define REGPOT0 0x00
#define REGPOT1 0x40
#define REGCTRL 0x80

//ControlRegisterConstants
#define CFGVOLATILE 0x04 //default on
#define CFGZEROCROS 0x02 //default on
#define CFG33STEPS  0x01 //default on

unsigned char encoder=128-(2*2);

signed char get_input(){
	static unsigned char cnt=0;
	static unsigned char in=0;
	signed char ret=0;

	//input process
	in<<=4;
	if(!PHASEA) in|=2;
	if(!PHASEB) in|=1;
	if(in&2) in^=1;

	//encoder detect
	if(in==0x12||in==0x32){
		if(encoder<(256-3)) encoder+=3;
	}
	if(in==0x10||in==0x30){
		if(encoder) encoder--;
	}
	
	//data process
	if(encoder&0x80){
		if(in==0x01) ret=1;
		if(in==0x03) ret=-1;
		if(in==0x10) ret=-1;
		if(in==0x30) ret=1;
	}else{
		if(in==0x01) ret=2;
		if(in==0x03) ret=-2;
		if(in==0x11||in==0x33) cnt++;
		else cnt=0;
		if(cnt==255){
			cnt=210;
			if(in==0x11) ret=2;
			if(in==0x33) ret=-2;
		}
	}
	return ret;
}

void main(){
	unsigned char vol,tmp,i;
	signed char in;
	unsigned int dcnt;	//EEPROM Write Delay Counter
	unsigned char analog;			//analog use flag

	//IO config
	CMCON = 0x07;
	TRISIO= 0x3F;
	GPPU  = 0;
	WPU   = 0x37;
	GPIO  = 0x3F;

	analog = 0;
	if(ANSEL==0x0F){		//Runing on 12F675
		WPU   = 0x35;
		ANSEL = 0x10;
		ADCON0= 0x01;		//Resistance on GP0 will checked
		for(i=0;i<10;i++){
			__delay_ms(1);
			GODONE= 1;
			while(GODONE){}
			tmp   = ADRESH;
			if( 8<tmp && tmp<230 ) analog++;
		}
		if(analog<8) analog=0;
	}

	__delay_ms(100);
		
	if(analog){
		ANSEL = 0x12;
		TRISIO= 0x36;
		WPU   = 0x34;
		GPIO  = 0x3F;
		ADCON0= 0x05;		//Voltage on GP1 will be sampled
		while(1){
			dcnt=0;
			for(i=0;i<16;i++){
				__delay_ms(1);
				GODONE= 1;
				while(GODONE){}
				dcnt += ADRESH;
			}
			tmp=255-(dcnt>>4);	//0-255	
			if(tmp-(vol<<2)>4){
				tmp>>=2;
			}else if((vol<<2)-tmp>1){
				tmp>>=2;
			}else{
				tmp=vol;
			}
			if(tmp!=vol){
				vol=tmp;
				i2c_Start();
				i2c_PutByte(I2CADDR|I2CWRITE);
				i2c_PutByte(REGPOT0|vol);
				i2c_PutByte(REGPOT1|vol);
				i2c_PutByte(REGCTRL|CFGVOLATILE|CFGZEROCROS);
				i2c_Stop();
			}
		}	
	}else{
		tmp=eeprom_read(0);	//Read volume
		if(tmp<64) vol=tmp;	//volume is valid
		else vol=20;			//default value
		
		//initialize
		WPU   = 0x37;
		ADCON0= 0;
		ANSEL = 0;
		in=0;
		dcnt=1;
	
		//main loop
		while(1){
			if(dcnt==1){
				i2c_Start();
				i2c_PutByte(I2CADDR|I2CWRITE);
				i2c_PutByte(REGPOT0|vol);
				i2c_PutByte(REGPOT1|vol);
				i2c_PutByte(REGCTRL|CFGVOLATILE|CFGZEROCROS);
				i2c_Stop();
			}
			in+=get_input();
			if(-2>=in){
				if(0<vol) vol--;
				in+=2;
				dcnt=1;
			}else if(2<=in){
				if(vol<63) vol++;
				in-=2;
				dcnt=1;
			}else{
				if(dcnt)	dcnt++;
			}
			if(dcnt==1000){
				tmp=eeprom_read(0);
				if(vol!=tmp) eeprom_write(0,vol);
				dcnt=0;
				in=0;
			}
			__delay_ms(1);
		}
	}	
}
